{% extends "base.html" %}
{% load i18n %}

{% block content %}
<h2>{% trans "Internal Server Error" %} (500)</h2>
<p>{% trans "There's been an error. It's been reported to the site administrators via e-mail and should be fixed shortly. Thanks for your patience." %}</p>
{% endblock %}
